/*
 * tivoRefCount.h
 *
 * This file contains declarations for TiVo's extensions
 * to gcc for checking our use of ref counted objects.
 *
 */

#ifndef __TIVO_REF_COUNT_H__
#define __TIVO_REF_COUNT_H__

#include "../tivoTypes.h"

const char* tivoGetId( tree node );
tivoBool tivoIsProbablyInstantiationOfRef( const char* name );
void tivoCheckCall( tree params );

// effects: if tivoRefCounting warnings are enabled,
//          causes warnings for any violations found
//          in this declaration
// note: this should be called on non-global declarations
//       to ensure that they are checked, because they're
//       not available at the end of parsing!
void tivoCheckDecl( tree decl );

// effects: if tivoRefCounting warnings are enabled,
//          causes warnings for any violations found
//          in the global namespace.
// note: for best results, this should be called once,
//       after the whole translation unit's been parsed.
void tivoCheckRefCountingRules();

// effects: if lhs is parameter, this checks that it's not
//          a pointer to a ref-counted object type.  call it
//          on parameters you see assignements to.
// requires: lhs is a PARM_DECL
void tivoCheckForAssignmentToRefCountedParameterType( tree lhs );

#endif // __TIVO_REF_COUNT_H__
