#include "tivoDeclIter.h"

#include <assert.h>
#include <stdlib.h>  // for getenv()

#include <stdio.h>
#include "cp/cp-tree.h"
#include "tivoTable.h"

#include <string.h>

// HACK: ugly reach into toplevel.c so we can temporarily
//       disable warnings_are_errors if we want to.
extern int warnings_are_errors;

extern int warn_tivo;

const char* tivoGetId( tree node );

/****************************************************************
 *
 * 
 *
 ****************************************************************/

static void tivoIterateOnDecl( tree node,
                               tivoTable_t visited,
                               tivoDeclIter_t iter )
{
    int nodeCode = ERROR_MARK;

    if ( node == NULL ) {
	// not sure when this happens...but it does!
	return;
    }

    /*
    fprintf( GetCgOut(), "*> tivoIterateOnDecl: %s(%s)\n",
	     tree_code_name[TREE_CODE(node)],
	     tivoGetId(node) );
    */

    // if already visiting or visited this node, don't repeat!
    if ( tivoTableContains( visited, node ) ) {
	return;
    }

    // add to table now to prevent loops!
    tivoTableAdd( visited, node );

    // get the node's code to dispatch below!
    nodeCode = TREE_CODE( node );

    /*
    fprintf( stderr, "---> tivoIterateOnDecl: %s(%s)\n",
	     tree_code_name[nodeCode],
	     tivoGetId(node) );
    */

    if ( nodeCode == VAR_DECL ) {
        iter( node, 1 );
    }
    else if ( nodeCode == FIELD_DECL ) {
        iter( node, 1 );
    }
    else if ( nodeCode == FUNCTION_DECL ) {
        //
        // the DECL_CHAIN on functions contains all the functions with
        // the same name.  this seems to be the only way to grab
        // overloaded methods.
        //

        tree pFunctionScan = node;
        for( ; pFunctionScan; pFunctionScan = DECL_CHAIN( pFunctionScan ) )
        {
            iter( pFunctionScan, 1 );
        }
    }
    else if ( nodeCode == TYPE_DECL ) {
        if ( iter( node, 1 ) )
        {
            tivoIterateOnDecl( TREE_TYPE(node), visited, iter );
        }
        iter( node, 0 );
    }
    else if ( nodeCode == RECORD_TYPE ) {
	tree scan = NULL;

        if ( iter( node, 1 ) )
        {
            if ( TYPE_LANG_SPECIFIC(node) != NULL ) {
                // check methods!
                for( scan = CLASSTYPE_METHOD_VEC(node); 
                     scan != NULL; 
                     scan = TREE_CHAIN(scan) ) {
                    tivoIterateOnDecl( scan, visited, iter );
                }
            }

            // check members!
            for( scan = TYPE_FIELDS(node);
                 scan != NULL;
                 scan = TREE_CHAIN(scan) ) {
                tivoIterateOnDecl( scan, visited, iter );
            }
        }
        iter( node, 0 );
    }
    else if ( nodeCode == TREE_VEC ) {
	int len = TREE_VEC_LENGTH(node);
	int i   = 0;

        if ( iter( node, 1 ) )
        {
            for( i=0; i < len; i++ ) {
                tree elt = TREE_VEC_ELT(node,i);
                tivoIterateOnDecl( elt, visited, iter );
            }
        }
        iter( node, 0 );
    }
    else {
	// fprintf( stderr, "     is *not* a handled type!\n" );
    }
}

static void tivoIterateOverChain( tree decls,
                                  tivoTable_t visited,
                                  tivoDeclIter_t iter )
{
    // first, fill an array with the decls 'cuz we want to visit them
    // in the proper order and they're stored reversed!

    const int numDecls = tivoCountChain( decls );
    const int numBytes = numDecls * sizeof(tree);
    tree* array  = (tree*) xmalloc( numBytes );
    tree  scan   = NULL;
    int i = 0;

    bzero( array, numBytes );
    
    for( scan = decls; scan != NULL; scan = TREE_CHAIN(scan) ) {
	array[i] = scan;
	i++;
    }

    // now visit the decls in the opposite order they're stored
    for( i=numDecls-1; i >= 0; i-- ) {
	tree aDecl = array[i];
	// debug_tree( aDecl );
	tivoIterateOnDecl( aDecl, visited, iter );
    }

    // and free the array!
    free( (char*) array );
}

/**********************************************************
 *
 * High-level checking routines!
 *
 **********************************************************/

extern int expanding_inline_function;

void tivoIterateOverGlobal( tivoDeclIter_t iter )
{
    tree decls = getdecls();
    tivoTable_t table = tivoTableCreate();
    
    tivoIterateOverChain( decls, table, iter );
    
    tivoTableFree( table );
}

void tivoIterateOver( tivoDeclIter_t iter, tree node )
{
    tivoTable_t table = tivoTableCreate();
    
    tivoIterateOnDecl( node, table, iter );
    
    tivoTableFree( table );
}

